#!/usr/bin/env python3
# coding : utf-8

import os
from pyrvtools.esx_types import Cluster, DataCenter, DataStore, Host, VirtualMachine
from pyrvtools.errors import PyRvtoolsError, ObjectNotFoundError, FileNonConformantError
from xlrd import open_workbook
from xlrd.sheet import Sheet


class PyRvtools(object):
    """ Extract useful information from an RVTools file """

    def __init__(self, filename: str):
        """
        Constructor
        :param filename: RVTools inventory file
        """

        if not os.path.isfile(filename):
            raise PyRvtoolsError('Incorrect filename: %s' % filename)

        if not os.access(filename, os.R_OK):
            raise PyRvtoolsError('Can\'t read file: %s' % filename)

        self._book = open_workbook(filename, on_demand=True)
        # self._health_check() # Slow with this method, full sheet load ?

    @staticmethod
    def get_columns_names(sheet: Sheet):
        """
        Return a dictionary with COLUMN_NAME:ID_COLUMN
        :param sheet: a Sheet object
        """

        mapping = {}
        for col_index in range(sheet.ncols):
            mapping[sheet.cell_value(0, col_index)] = col_index
        return mapping

    def _get_names(self, sheet_name: str, value_name: str):
        """
        Generator - Return a range of values from a column
        :param sheet_name: Name of the sheet to parse
        :param value_name: Value you are looking for
        """
        sheet = self._book.sheet_by_name(sheet_name)
        mapping = self.get_columns_names(sheet)
        for idx in range(sheet.nrows):
            if idx == 0:
                continue
            yield sheet.cell_value(idx, mapping[value_name])

    def _health_check(self):
        """ Do some health check before go ahead """

        all_tabs = []
        needed_tabs = ['tabvInfo', 'tabvDisk', 'tabvPartition',
                       'tabvHost', 'tabvHBA', 'tabvDatastore']

        for sheet in self._book.sheets():
            all_tabs.append(sheet.name)

        if not set(needed_tabs).issubset(set(all_tabs)):
            msg = 'The file is not a RVTools file'
            raise FileNonConformantError(msg)

    def get_clusters(self):
        """
        Generator - return a list of Cluster objects
        :return Cluster
        """

        for cluster in set(self._get_names('tabvHost', 'Cluster')):
            if cluster:
                yield Cluster(self._book, cluster)

    def get_clusters_by_name(self, name):
        """
        Search a Cluster object and return it
        :param name: Name of that Cluster
        """

        found = None
        for cluster in self._get_names('tabvHost', 'Cluster'):
            if cluster == name:
                found = Cluster(self._book, cluster)
                break

        if not found:
            raise ObjectNotFoundError('Cluster %s not found' % name)

        return found

    def get_datacenters(self):
        """
        Generator - return a list of DataCenter objects
        :return DataCenter
        """

        for datacenter in set(self._get_names('tabvHost', 'Datacenter')):
            if datacenter:
                yield DataCenter(self._book, datacenter)

    def get_datacenter_by_name(self, name):
        """
        Search a DataCenter object and return it
        :param name: Name of that DataCenter
        """

        found = None
        for datacenter in self._get_names('tabvHost', 'Datacenter'):
            if datacenter == name:
                found = DataCenter(self._book, datacenter)
                break

        if not found:
            raise ObjectNotFoundError('Datacenter %s not found' % name)

        return found

    def get_datastores(self):
        """
        Generator - return a list of DataStore objects
        :return DataStore
        """

        for datastore in self._get_names('tabvDatastore', 'Name'):
            yield DataStore(self._book, datastore)

    def get_datastore_by_name(self, name):
        """
        Search a DataStore object and return it
        :param name: Name of that DataStore
        """

        found = None
        for datastore in self._get_names('tabvDatastore', 'Name'):
            if datastore == name:
                found = DataStore(self._book, datastore)
                break

        if not found:
            raise ObjectNotFoundError('Datastore %s not found' % name)
        return found

    def get_hosts(self):
        """
        Generator - return a list of Host objects
        :return Host
        """

        for host in self._get_names('tabvHost', 'Host'):
            yield Host(self._book, host)

    def get_host_by_name(self, name):
        """
        Search a Host object and return it
        :param name: Name of that Host
        """

        found = None
        for host in self._get_names('tabvHost', 'Host'):
            if host == name:
                found = Host(self._book, host)
                break

        if not found:
            raise ObjectNotFoundError('Host %s not found' % name)
        return found

    def get_vm(self):
        """
        Generator - return a list of VirtualMachine objects
        :return VirtualMachine
        """

        for vm in self._get_names('tabvInfo', 'VM'):
            yield VirtualMachine(self._book, vm)

    def get_vm_by_name(self, name):
        """
        Search a VirtualMachine object and return it
        :param name: Name of that VirtualMachine
        """

        found = None
        for vm in self._get_names('tabvInfo', 'VM'):
            if vm == name:
                found = VirtualMachine(self._book, vm)
                break

        if not found:
            raise ObjectNotFoundError('VirtualMachine %s not found' % name)

        return found



#!/usr/bin/env python3
# coding : utf-8

import pyrvtools.pyrvtools
from datetime import datetime
from xlrd import book
from xlrd.sheet import Sheet


class ESXBase(object):
    """ Super class of every ESX object """

    def __init__(self, workbook: book, name: str):
        """
        Constructor
        :param workbook: a workbook XLRD
        :param name: Name of that ESX object
        """
        self._book = workbook
        self._name = name
        self._data = None
        self._sheet = None
        self._column = None

    def __eq__(self, other):
        compare = False
        if isinstance(other, str):
            compare = self.name == other
        if isinstance(other, self.__class__):
            compare = self.name == other.name
        return compare

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self._name)

    def __str__(self):
        return self._name

    # only_one : just to speed up the search in case of unique value

    def _search(self, sheet: Sheet, column: str, target: str, only_one=False):
        """
        Generic search method
        :param sheet: A Sheet XLRD Object where perform the search
        :param column: Name of the column where perform the search
        :param target: Value to find
        :param only_one: (boolean) search one (true) or multiple values (false)
        :return dict: a dictionary with a complete row of data
        """
        target_rows = []
        column_names = pyrvtools.pyrvtools.PyRvtools.get_columns_names(sheet)
        for row_number in range(sheet.nrows):
            if sheet.cell_value(row_number, column_names[column]) == target:
                target_rows.append(row_number)

                if target_rows and only_one:
                    break

        all_answer = []
        for target_row in target_rows:
            one_answer = {}
            for col_name, col_number in column_names.items():
                one_answer[col_name] = sheet.cell_value(target_row, col_number)
            all_answer.append(one_answer)
        return all_answer

    def _search_one_value(self, item):
        """
        Search only one occurrence of a value in the embedded sheet
        :param item: item to find
        """
        if not self._data:
            self._data = self._search(self._sheet, self._column,
                                      self._name, only_one=True)
        return self._data[0][item]

    @property
    def name(self):
        return self._name


class Cluster(ESXBase):
    """ Object that's represent a vSphere Cluster """

    def __init__(self, workbook: book, name: str):
        """
        Contructor
        :param workbook: a XLRD workbook
        :param name: name of the Cluster
        """
        super().__init__(workbook, name)
        self._sheet = self._book.sheet_by_name('tabvHost')
        self._column = 'Cluster'
        self._hosts = []

    @property
    def datacenter(self):
        return self._search_one_value('Datacenter')

    @property
    def hosts(self):
        if not self._hosts:
            data = self._search(sheet=self._sheet, column='Cluster',
                                target=self._name)
            for host in data:
                self._hosts.append(Host(self._book, host['Host']))

        return self._hosts


class DataCenter(ESXBase):
    """ Object that's represent a vSphere DataCenter """

    def __init__(self, workbook: book, name: str):
        """
        Contructor
        :param workbook: a XLRD workbook
        :param name: name of that DataCenter
        """
        super().__init__(workbook, name)
        self._sheet = self._book.sheet_by_name('tabvHost')
        self._column = 'Datacenter'
        self._clusters = []
        self._hosts = []

    @property
    def clusters(self):
        if not self._clusters:
            data = self._search(sheet=self._sheet, column='Datacenter',
                                target=self._name)
            for cluster in data:
                if any(i.name == cluster['Cluster'] for i in self._clusters):
                    continue

                if not cluster['Cluster']:
                    continue

                self._clusters.append(Cluster(self._book, cluster['Cluster']))

        return self._clusters

    @property
    def hosts(self):
        if not self._hosts:
            data = self._search(sheet=self._sheet, column='Datacenter',
                                target=self._name)
            for host in data:
                self._hosts.append(Host(self._book, host['Host']))

        return self._hosts


class DataStore(ESXBase):
    """ Object that's represent a vSphere DataStore """

    def __init__(self, workbook: book, name: str):
        """
        Constructor
        :param workbook: a XLRD workbook
        :param name: name of that DataStore
        """
        super().__init__(workbook, name)
        self._sheet = self._book.sheet_by_name('tabvDatastore')
        self._column = 'Name'
        self._hosts = []

    @property
    def capacity_mb(self):
        return int(self._search_one_value('Capacity MB'))

    @property
    def free_mb(self):
        return int(self._search_one_value('Free MB'))

    @property
    def free_percent(self):
        return int(self._search_one_value('Free %'))

    @property
    def hosts(self):
        if not self._hosts:
            hosts = self._search_one_value('Hosts')
            for host in hosts.split(', '):
                self._hosts.append(Host(self._book, host))

        return self._hosts

    @property
    def inuse_mb(self):
        return int(self._search_one_value('In Use MB'))

    @property
    def naa(self):
        naa = self._search_one_value('Address')
        naa = naa.split('.')[1] if naa and naa.startswith('naa.') else ''
        return naa

    @property
    def number_of_hosts(self):
        return int(self._search_one_value('# Hosts'))

    @property
    def number_of_vms(self):
        return int(self._search_one_value('# VMs'))

    @property
    def provisioned_mb(self):
        return int(self._search_one_value('Provisioned MB'))

    @property
    def sioc_enable(self):
        return bool(self._search_one_value('SIOC enabled'))

    @property
    def type(self):
        return self._search_one_value('Type')

    @property
    def version(self):
        return str(self._search_one_value('Major Version'))


class HBA(object):
    """ Tiny object that's represent an HBA adapter """

    def __init__(self, **kwargs):
        self._hba = kwargs

    def __repr__(self):
        return 'HBA(%s)' % self._hba['Device']

    def __str__(self):
        return self._hba['Device']

    @property
    def device(self):
        return self._hba['Device']

    @property
    def status(self):
        return self._hba['Status']

    @property
    def type(self):
        return self._hba['Type']

    @property
    def driver(self):
        return self._hba['Driver']

    @property
    def model(self):
        return self._hba['Model']

    @property
    def wwn(self):
        wwn = None
        if self.type == 'Fibre Channel':
            wwn = self._hba['WWN'].split(' ')
        return wwn


class Host(ESXBase):
    """ Object that's represent a vSphere DataStore """

    def __init__(self, workbook: book, name: str):
        """
        Constructor
        :param workbook: a XLRD workbook
        :param name: name of that Host
        """
        super().__init__(workbook, name)
        self._sheet = self._book.sheet_by_name('tabvHost')
        self._column = 'Host'
        self._hba = []
        self._vms = []

    @property
    def boot_time(self):
        date = self._search_one_value('Boot time')
        return datetime.strptime(date, '%d/%m/%Y %H:%M:%S')

    @property
    def cluster(self):
        return self._search_one_value('Cluster')

    @property
    def cpu_usage_percent(self):
        return int(self._search_one_value('CPU usage %'))

    @property
    def datacenter(self):
        return self._search_one_value('Datacenter')

    @property
    def esx_version(self):
        return self._search_one_value('ESX Version')

    @property
    def hba(self):
        if not self._hba:
            data = self._search(sheet=self._book.sheet_by_name('tabvHBA'),
                                column='Host',
                                target=self._name)
            for one_hba in data:
                self._hba.append(HBA(**one_hba))

        return self._hba

    @property
    def memory_mb(self):
        return int(self._search_one_value('# Memory'))

    @property
    def memory_usage_percent(self):
        return int(self._search_one_value('Memory usage %'))

    @property
    def model(self):
        return self._search_one_value('Model')

    @property
    def number_of_cores(self):
        return int(self._search_one_value('# Cores'))

    @property
    def number_of_cpu(self):
        return int(self._search_one_value('# CPU'))

    @property
    def number_of_vcpu(self):
        return int(self._search_one_value('# vCPUs'))

    @property
    def number_of_vm(self):
        return int(self._search_one_value('# VMs'))

    @property
    def vm(self):
        if not self._vms:
            data = self._search(sheet=self._book.sheet_by_name('tabvInfo'),
                                column='Host',
                                target=self._name)
            for vm in data:
                self._vms.append(VirtualMachine(self._book, vm['VM']))

        return self._vms


class VirtualMachine(ESXBase):
    """ Object that's represent a vSphere Virtual Machine """

    def __init__(self, workbook: book, name: str):
        """
        Constructor
        :param workbook: a XLRD workbook
        :param name: name of that VirtualMachine
        """
        super().__init__(workbook, name)
        self._sheet = self._book.sheet_by_name('tabvInfo')
        self._column = 'VM'
        self._vdisks = []
        self._vpartitions = []
        self._vnetworks = []

    @property
    def cluster(self):
        return Cluster(self._book, self._search_one_value('Cluster'))

    @property
    def cpu(self):
        return int(self._search_one_value('CPUs'))

    @property
    def datacenter(self):
        return DataCenter(self._book, self._search_one_value('Datacenter'))

    @property
    def datastore(self):
        path = self._search_one_value('Path')
        return DataStore(self._book, path[path.find('[')+1:path.find(']')])

    @property
    def host(self):
        return Host(self._book, self._search_one_value('Host'))

    @property
    def inuse_mb(self):
        return int(self._search_one_value('In Use MB'))

    @property
    def memory(self):
        return int(self._search_one_value('Memory'))

    @property
    def os(self):
        return self._search_one_value('OS')

    @property
    def power_on(self):
        timestamp = None
        date = self._search_one_value('PowerOn')
        if date:
            timestamp = datetime.strptime(date, '%d/%m/%Y %H:%M:%S')
        return timestamp

    @property
    def power_state(self):
        return self._search_one_value('Powerstate')

    @property
    def provisioned_mb(self):
        return int(self._search_one_value('Provisioned MB'))

    @property
    def unshared_mb(self):
        return int(self._search_one_value('Unshared MB'))

    @property
    def vmdk(self):
        if not self._vdisks:
            data = self._search(sheet=self._book.sheet_by_name('tabvDisk'),
                                column='VM',
                                target=self._name)

            for disk in data:
                self._vdisks.append(VDisk(book=self._book, **disk))

        return self._vdisks

    @property
    def vnetwork(self):
        if not self._vnetworks:
            data = self._search(sheet=self._book.sheet_by_name('tabvNetwork'),
                                column='VM',
                                target=self._name)

            for network in data:
                self._vnetworks.append(VNetwork(**network))

        return self._vnetworks

    @property
    def vpartition(self):
        if not self._vpartitions:
            data = self._search(sheet=self._book.sheet_by_name('tabvPartition'),
                                column='VM',
                                target=self._name)

            for disk in data:
                self._vpartitions.append(VPartition(**disk))

        return self._vpartitions


class VDisk(object):
    """ Tiny object that's represent a VMDK disk """

    def __init__(self, **kwargs):
        self._data = kwargs
        self._book = kwargs['book']

    def __repr__(self):
        return 'VMDK(%s)' % self._data['Disk']

    def __str__(self):
        return self._data['Disk']

    @property
    def capacity_mb(self):
        return int(self._data['Capacity MB'])

    @property
    def datastore(self):
        path = self._data['Path']
        return DataStore(self._book, path[path.find('[')+1:path.find(']')])

    @property
    def eagerly_scrub(self):
        return bool(self._data['Eagerly Scrub'])

    @property
    def thin(self):
        return bool(self._data['Thin'])


class VNetwork(object):
    """ Tiny object that's represent a virtual network adapter """

    def __init__(self, **kwargs):
        self._data = kwargs

    def __repr__(self):
        return 'VNetwork(%s,%s,%s)' % (self._data['VM'],
                                       self._data['Adapter'],
                                       self._data['Network'])

    def __str__(self):
        return '%s,%s,%s' % (self._data['VM'],
                             self._data['Adapter'],
                             self._data['Network'])

    @property
    def adapter(self):
        return self._data['Adapter']

    @property
    def connected(self):
        return bool(self._data['Connected'])

    @property
    def ip_address(self):
        ip = []
        if self._data['IP Address']:
            if not self._data['IP Address'] == 'unknown':
                ip = self._data['IP Address'].split(', ')

        return ip

    @property
    def mac_address(self):
        return self._data['Mac Address']

    @property
    def network(self):
        return self._data['Network'] if self._data['Network'] else None

    @property
    def powerstate(self):
        return self._data['Powerstate']

    @property
    def switch(self):
        return self._data['Switch']


class VPartition(object):
    """ Tiny object that's represent the data (partitions) inside a VM """

    def __init__(self, **kwargs):
        self._data = kwargs

    def __repr__(self):
        return 'VPartition(%s)' % self._data['Disk']

    def __str__(self):
        return self._data['Disk']

    @property
    def capacity_mb(self):
        return int(self._data['Capacity MB'])

    @property
    def disk(self):
        return self._data['Disk']

    @property
    def free_mb(self):
        return int(self._data['Free MB'])

    @property
    def free_percent(self):
        return int(self._data['Free % '])



from setuptools import setup

def readme():
    with open('README.rst') as fh:
        return fh.read()

setup(
    name='pyrvtools',
    version='1.0.1',
    packages=['pyrvtools'],
    install_requires=['xlrd'],
    url='https://github.com/jbrt/pyrvtools',
    license='GPL',
    author='Julien B.',
    author_email='julien@toshokan.fr',
    description='Extract useful information from an RVTools ESX inventory file',
    long_description=readme(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)
